@@ -36,11 +36,12 @@ from .typeoverloads import FilePath
 # For ruamel yaml
 # -----
 from ruamel.yaml import YAML, YAMLError
-yaml = YAML()
+dumper = YAML()
+loader = YAML(typ='safe')
 def yamlLoad(stream):
-  return yaml.load(stream)
+  return loader.load(stream)
 def yamlDump(data, stream):
-  return yaml.dump(data, stream)
+  return dumper.dump(data, stream)
 
 
 def mproc_apply(func, iterLst, descr='', iterArgPos=0, extraArgs=(), showProgress=True, applyAsync=True, total=None, pool=None):
@@ -641,4 +642,31 @@ def hierarchicalUpdate(curDict: dict, other: dict, replaceLists=False):
       curVal.extend(v)
     else:
       curDict[k] = v
-  return curDict
\ No newline at end of file
+  return curDict
+
+def timedExec(updateFunc: t.Union[t.Generator, t.Callable], interval: int,
+              argLst: t.Sequence=None, stopCond: t.Callable=None):
+  """
+  Iterates over an argument list (or generator) and executes the next action every
+  `interval` ms. Useful for updating GUI elements periodically, e.g. showing a list
+  of images one at a time
+  """
+  timer = QtCore.QTimer()
+  if argLst is None:
+    # updateFunc must be a generator
+    argLst = updateFunc()
+    updateFunc = lambda arg: None
+  if stopCond is None:
+    stopCond = lambda: False
+  argLst = iter(argLst)
+  def update():
+    if stopCond():
+      timer.stop()
+    try:
+      arg = next(argLst)
+      updateFunc(arg)
+    except StopIteration:
+      timer.stop()
+  timer.timeout.connect(update)
+  timer.start(interval)
+  return timer
\ No newline at end of file
