@@ -4,23 +4,25 @@ but there are many flaws and the project is not under active development. It is
 also less pythonic than it could be, using functions where properties are more
 appropriate.
 """
+from __future__ import annotations
 import contextlib
 import copy
+import typing as t
+from contextlib import contextmanager
 from warnings import warn
 from collections import deque
 from functools import wraps
-from typing import Callable, Generator, Deque, Union, Type, Any, List
 
 from typing_extensions import Protocol
 
-__all__ = ['ActionStack', 'Action']
+__all__ = ['ActionStack', 'Action', 'DeferredActionStackMixin']
 
 # _generatorCallable = Callable[[...], Union[Generator, Any]]
 class Action:
   """
   This represents an action which can be done and undone.
   """
-  def __init__(self, generator: Callable[..., Union[Generator, Any]], args:tuple=None,
+  def __init__(self, generator: t.Callable[..., t.Union[t.Generator, t.Any]], args:tuple=None,
                kwargs:dict=None, descr: str=None, treatAsUndo=False):
     if args is None:
       args = []
@@ -39,7 +41,7 @@ class Action:
       # Need to init runner for when backward is called
       self._runner = self._generator(*args, **kwargs)
 
-  def reassignBackward(self, backwardFn: Callable[..., Any], backwardArgs=(),
+  def reassignBackward(self, backwardFn: t.Callable[..., t.Any], backwardArgs=(),
                        backwardKwargs=None):
 
     if backwardKwargs is None:
@@ -84,7 +86,7 @@ class Action:
     return ret
 
 class EMPTY: pass
-EmptyType = Type[EMPTY]
+EmptyType = t.Type[EMPTY]
 _FRONT = -1
 _BACK = 0
 
@@ -92,7 +94,7 @@ class Appendable(Protocol):
   def append(self):
     raise NotImplementedError
 
-def gracefulNext(generator: Generator):
+def gracefulNext(generator: t.Generator):
   try:
     return next(generator)
   except StopIteration as ex:
@@ -112,10 +114,10 @@ class ActionStack:
   """
 
   def __init__(self, maxlen:int=50):
-    self.actions: Deque[Action] = deque(maxlen=maxlen)
+    self.actions: t.Deque[Action] = deque(maxlen=maxlen)
     self._curReceiver = self.actions
-    self._savepoint: Union[EmptyType, Action] = EMPTY
-    self.stackChangedCallbacks: List[Callable] = []
+    self._savepoint: t.Union[EmptyType, Action] = EMPTY
+    self.stackChangedCallbacks: t.List[t.Callable] = []
 
   @contextlib.contextmanager
   def group(self, descr: str=None, flushUnusedRedos=False):
@@ -124,7 +126,7 @@ class ActionStack:
     All actions which occur within the group will be undone by a single call
     of `stack.undo`.
     """
-    newActBuffer: Deque[Action] = deque()
+    newActBuffer: t.Deque[Action] = deque()
     with _BufferOverride(self, newActBuffer):
       yield
     def grpAct():
@@ -148,13 +150,14 @@ class ActionStack:
     :param descr: Description of this action, e.g. "add components", etc.
     :param asGroup: If *True* assumes this undoable function is a composition
       of other undoable functions. This is a simple alias for
+      >>> stack: ActionStack, func: t.Callable, args: tuple, kwargs: dict
       >>> with stack.group('descr', flushUnusedRedos=True):
       >>>  func(*args, **kwargs)
     :param copyArgs: Whether to make a copy of the arguments used for the undo
       function. This is useful for functions where the input argument is modified
       during the function call. WARNING: UNTESTED
     """
-    def decorator(generatorFn: Callable[..., Generator]):
+    def decorator(generatorFn: t.Callable[..., t.Generator]):
       nonlocal descr
       if descr is None:
         descr = generatorFn.__name__
@@ -229,7 +232,7 @@ class ActionStack:
     """
     if maxLength == self.actions.maxlen:
       return
-    newDeque: Deque[Action] = deque(maxlen=maxLength)
+    newDeque: t.Deque[Action] = deque(maxlen=maxLength)
     newDeque.extend(self.actions)
     receiverNeedsReset = True if self._curReceiver is self.actions else False
     self.actions = newDeque
@@ -332,4 +335,83 @@ class _BufferOverride:
     return self
 
   def __exit__(self, exc_type, exc_val, exc_tb):
-    self.stack._curReceiver = self.oldStackActions
\ No newline at end of file
+    self.stack._curReceiver = self.oldStackActions
+
+
+class DeferredActionStackMixin:
+  """
+  When registering undoable actions of a class with an action stack, it is impossible to use the reference stack's
+  decorator since the stack isn't constructed yet. Moreover, creating a class-level default stack and registering all
+  actions to it will force all instances of that class to share the same undo buffer. this deferred undoable solves the
+  problem. It will record which functions the user wants to register as undoable, and the initialization procedure of
+  ActionStackMixin will complete this registration as instances are created. That way, instances of the same class
+  have the option of sharing undo buffers, but aren't forced to do so.
+
+  This has the same effect as registering undoable actions in the __init__ body, e.g.:
+
+  ```python
+    def __init__(self, ...):
+      self.func1 = self.actionStack.undoable('descr')(func1)
+
+      self.func2 = self.actionStack.undoable()(func2)
+
+      # ...
+  ```
+
+  Note that deferred action stacks colocate generator functions with their "undoable" registration to avoid confusion,
+  promote reuse, etc.
+  """
+  actionStack: ActionStack = None
+  _deferredUndoables = {}
+
+  class undoable:
+    """
+    Intended for use solely as a function decorator where ActionStack.undoable would be used. It's deferred nature,
+    though, means no stack object must exist at the time of registering the undoable function. Instead, a stack is
+    provided for this function on creationg of the ActionStackMixin class.
+    """
+    def __set_name__(self, owner: t.Type[DeferredActionStackMixin], name):
+      if not issubclass(owner, DeferredActionStackMixin):
+        raise TypeError('Can only use deferred undoable on ActionStackMixin classes')
+      # At this point, we at some point had acecss to the decorated function, deco description/args, and finally
+      # we have the owning class. Add a deferred undoable to be later registered. This is a bit tricky, since a plain
+      # reference to _deferredUndoables may share references to other classes (which we don't want). Using copy()
+      # ensures the default stack is unique to *this* class, preventing one class from registering methods of another.
+      defUndos = owner._deferredUndoables.copy()
+      defUndos[name] = self.regArgs
+      owner._deferredUndoables = defUndos
+
+    def __init__(self, *args, **kwargs):
+      self.regArgs = (args, kwargs)
+      super().__init__()
+
+    def __get__(self, instance, owner):
+      return self.func.__get__(instance, owner)
+
+    def __call__(self, func):
+      self.func = func
+      return self
+
+  def __new__(cls, *args, **kwargs):
+    stack = cls.actionStack if cls.actionStack else ActionStack()
+    obj = super().__new__(cls, *args, **kwargs)
+    for funcName, (regArgs, regKwargs) in cls._deferredUndoables.items():
+      bound = getattr(obj, funcName)
+      registered = stack.undoable(*regArgs, **regKwargs)(bound)
+      setattr(obj, funcName, registered)
+      obj.actionStack = stack
+    return obj
+
+  @classmethod
+  @contextmanager
+  def setStack(cls, stack: ActionStack):
+    """
+    Allows the code instantiating deferred stack objects to set the desired undo stack for that new instance. This is
+    reverted back the the original value upon exiting the context manager
+    :param stack: Stack to which undo actions should be registered
+    :return: Context manager that resets the default registration stack on exit
+    """
+    oldStack = cls.actionStack
+    cls.actionStack = stack
+    yield
+    cls.actionStack = oldStack
\ No newline at end of file
