@@ -1,3 +1,4 @@
+import inspect
 import typing as t
 import weakref
 from contextlib import contextmanager
@@ -7,7 +8,6 @@ from pathlib import Path
 from pyqtgraph.Qt import QtCore, QtWidgets
 from pyqtgraph.parametertree import Parameter
 from pyqtgraph.parametertree.parameterTypes import GroupParameter
-import pyqtgraph as pg
 
 from utilitys.widgets import EasyWidget, DummySignal
 from . import PrjParam
@@ -17,7 +17,8 @@ from ..processing import *
 from ..typeoverloads import FilePath
 
 __all__ = [
-  'RunOpts', 'ParamEditor', 'ParamEditorDockGrouping', 'EditorPropsMixin', 'SPAWNED_EDITORS'
+  'RunOpts', 'ParamEditor', 'ParamEditorDockGrouping', 'EditorPropsMixin', 'SPAWNED_EDITORS',
+  'ParamContainer'
 ]
 
 Signal = QtCore.Signal
@@ -47,6 +48,37 @@ def _mkRunDict(proc: ProcessStage, btnOpts: t.Union[PrjParam, dict]):
   return defaultBtnOpts
 
 
+class ParamContainer:
+  """
+  Utility for exposing dict-like behavior to the user for inserting and querying parameter objects. Behaves amost
+  like a pyqtgraph parameter, with the exception that new parameter "children" can be added through the dict
+  interface
+  """
+  def __init__(self):
+    self.params = {}
+
+  def __setitem__(self, key, value):
+    if key not in self.params:
+      # New parameter
+      self.params[key] = value
+    else:
+      self.params[key].setValue(value)
+
+  def __getitem__(self, item):
+    item = self.params[item]
+    try:
+      return item.value()
+    except AttributeError:
+      return item
+
+  def get(self, key, default=None):
+    try:
+      return self[key]
+    except KeyError:
+      return default
+
+  def keys(self):
+    return self.params.keys()
 """
 Eventually, it would be nice to implemenet a global search bar that can find/modify
 any action, shortcut, etc. from any parameter. This tracker is an easy way to fascilitate
@@ -347,8 +379,7 @@ class ParamEditor(QtWidgets.QDockWidget):
       return
     filename.unlink()
 
-  def registerProps(self, constParams: t.List[PrjParam], namePath:t.Sequence[str]=(),
-                     asProperty=True, **extraOpts):
+  def registerProps(self, constParams: t.List[PrjParam], *args, **kwargs):
     """
     Registers a list of proerties and returns an array of each. For parameter descriptions,
     see :func:`PrjParamEditor.registerProp`.
@@ -356,11 +387,13 @@ class ParamEditor(QtWidgets.QDockWidget):
     outProps = []
     with self.params.treeChangeBlocker():
       for param in constParams:
-        outProps.append(self.registerProp(param, namePath, asProperty, **extraOpts))
+        outProps.append(self.registerProp(param, *args, **kwargs))
     return outProps
 
   def registerProp(self, constParam: PrjParam=None, namePath: t.Sequence[str]=(),
-                   asProperty=True, overrideBasePath: t.Sequence[str]=None, **etxraOpts):
+                   asProperty=True, overrideBasePath: t.Sequence[str]=None,
+                   container: ParamContainer=None,
+                   **etxraOpts):
     """
     Registers a property defined by *constParam* that will appear in the respective
     parameter editor.
@@ -376,6 +409,7 @@ class ParamEditor(QtWidgets.QDockWidget):
       x = registerProp(..., asProperty=False); myVal = x.value()
     :param overrideBasePath: Whether to use the base path specified by ParamEditor._baseRegisterPath
       (if *None*) or this specified override
+    :param container: Optional container in which to insert this parameter
     :param etxraOpts: Extra options passed directly to the created :class:`pyqtgraph.Parameter`
 
     :return: Property bound to this value in the parameter editor
@@ -390,6 +424,8 @@ class ParamEditor(QtWidgets.QDockWidget):
     paramForEditor = fns.getParamChild(self.params, *namePath, chOpts=paramOpts)
 
     self.registeredPrjParams.append(constParam)
+    if container is not None:
+      container[constParam] = paramForEditor
     if not asProperty:
       return paramForEditor
 
@@ -772,21 +808,48 @@ class ParamEditorDockGrouping(QtWidgets.QDockWidget):
 class EditorPropsMixin:
   __groupingName__: str = None
 
+  opts = {}
+  """Options passed to kwargs during __initEditorParams__"""
+
   REGISTERED_GROUPINGS = set()
+
   def __new__(cls, *args, **kwargs):
+    # ParamEditor case
+    if hasattr(cls, 'name') and cls.name is not None and cls.__groupingName__ is None:
+      # The '&' character can be used in Qt to signal a shortcut, which is used internally
+      # by plugins. When these plugins register shortcuts, the group name will contain
+      # ampersands which shouldn't show up in human readable menus
+      cls.__groupingName__ = cls.name.replace('&', '')
     if cls.__groupingName__ is None:
       cls.__groupingName__ = fns.nameFormatter(cls.__name__)
-    if cls not in cls.REGISTERED_GROUPINGS:
-      basePath = (cls.__groupingName__,)
-      if basePath[0] == '':
-        basePath = ()
-      with ParamEditor.setBaseRegisterPath(*basePath):
-        cls.__initEditorParams__()
-      cls.REGISTERED_GROUPINGS.add(cls)
-    return super().__new__(cls, *args, **kwargs)
+    basePath = (cls.__groupingName__,)
+    if basePath[0] == '':
+      basePath = ()
+    obj = super().__new__(cls, *args, **kwargs)
+    with ParamEditor.setBaseRegisterPath(*basePath):
+      # Check if kwargs are desired
+      try:
+        inspect.signature(obj.__initEditorParams__).bind(**cls.opts)
+      except TypeError:
+        # Call without these kwargs
+        obj.__initEditorParams__()
+      else:
+        obj.__initEditorParams__(**cls.opts)
+    return obj
 
   @classmethod
-  def __initEditorParams__(cls):
+  @contextmanager
+  def setOpts(cls, **opts):
+    """
+    sets the optios accociated with calls to __initEditorParams__ for instances of this class
+    created inside the context manager
+    """
+    oldOpts = cls.opts.copy()
+    cls.opts.update(opts)
+    yield
+    cls.opts = oldOpts
+
+  def __initEditorParams__(self, **kwargs):
     pass
 
 SPAWNED_EDITORS: t.List[ParamEditor] = []
\ No newline at end of file
