@@ -330,13 +330,19 @@ class AtomicProcess(ProcessStage):
     missingKeys = (requiredKeyFmt - setKeys)
     if missingKeys and not graceful:
       # Some required keys were not given
-      raise KeyError(f'Missing Following keys from {self}: {missingKeys}')
+      raise KeyError(f'Missing following keys from {self}: {missingKeys}')
 
   def saveState(self, includeDefaults=False, includeMeta=False, **metaFilterOpts):
     def keepCond(k, v):
-      return (k in self.input.hyperParamKeys or k not in self.defaultInput) \
+      # Can fail with unsavable types (e.g. numpy arrays)
+      # TODO: Determine appropriate response
+      # noinspection PyBroadException
+      try:
+        return (k in self.input.hyperParamKeys or k not in self.defaultInput) \
               and (includeDefaults or v != self.defaultInput.get(k)) \
               and v is not self.input.FROM_PREV_IO
+      except Exception:
+        return False
     saveIo = {k: v for k, v in self.input.items() if keepCond(k, v)}
     if not saveIo:
       state = self.name
